classdef lagrange12D < finiteElements2D
    % class for build Lagrange L1 FE in 2D
    % Inherits everything from abstract class finiteElements
    % Only different Matrices were defined as constant props.
    
    
    properties(Constant)
        % local matrices, resp. vectors. We use the vector form to use
        % later the sparse function to create the matrices... cf. the code
        % in assema etc.
        
        % stiff
        S1 = 0.5*[ 1
                  -1
                   0
                  -1
                   1
                   0
                   0
                   0
                   0];
               
        S2 = 0.5*[ 2 
                  -1 
	              -1 
                  -1 
                   0 
                   1  
                  -1
                   1
                   0];
              
        S3 = 0.5*[ 1  
                   0  
	              -1  
                   0 
                   0  
                   0  
                  -1
                   0
                   1];
              
        % mass
        M = 1/24*[2  
	               1  
	               1  
                   1   
                   2  
                   1  
                   1
                   1 
                   2];
               
        % RHS vector
        F = 1/6*[1
                  1
                  1]; 
              
        % Matrices convection terms 
        % for use by "sparse" in vector form
        C1 = [-1/6   
           -1/6   
           -1/6  
            1/6
            1/6
            1/6
            0
            0
            0];
        
        C2 = [-1/6   
           -1/6   
           -1/6 
            0
            0
            0
            1/6
            1/6
            1/6]; 
        
      
        % sparsity constant for allocating memory
        sparsityConstant = 5;
        
        % index vector for selecting coordinates
        idx = 1:3;
    end
    
    methods(Static,Access=public)   
        function [idxvec0,idxvec1,idxvec2] = makeIndex(idx,np)
            idxvec0 = reshape(idx,1,np*3);
            idxvec1 = reshape([idx;idx;idx],1,np*9);
            idxvec2 =  reshape([idx(1,:);idx(1,:);idx(1,:);...
                idx(2,:); idx(2,:); idx(2,:);...
                idx(3,:);idx(3,:);idx(3,:)],1,9*np);     
        end  
    end
end