%%
% PLOTBRA : plot branch of p-structure.
%
%  plotbra(p,varargin)
%  varargin = window number, component number (see below), [option value pairs]
% 
% Component number is counting from the non-user data of bradat(p) onward, 
% e.g. cmp=0 is last component of bradat(p) and cmp can be negative.
%
% Option value pairs: 
%
% *      'lwun' : Linewidth of unstable Sol.
% *      'lwst' : Linewidth of stable Sol.
% *      'tyun' : Linetype of unstable Sol.
% *      'tyst' : Linetype of stable Sol.
% *      'ms'   : Markersize (bif.points)
% *      'rms'  : Markersize (reg. points)
% *      'fms'  : Markersize (fold points)
% *      'lms'  : Markersize (labels)
% *      'inc'  : Show point markers stepping through all points with given increment 
% *      'lab'  : Place markers and labels for given label-list
% *      'labi' : As 'lab', but step through all labels in branch with given increment
% *      'labo' : Place label with given x- and y-offsets from location on branch
% *      'fs'   : Fontsize for labels and axesfs
% *      'lfs'  : Override fontsize for labels, if zero no labels (only markers)
% *      'fp'   : First point to be considered
% *      'lp'   : Last point to be considered
% *      'cl'   : Color, see option in plot
%
% See also stanparam, plot, plotbraf, bradat.
function plotbra(p,varargin)
if(nargin==1) % predefined plotting
    wnr=p.plot.brafig; cmp=p.plot.bpcmp;  
    varargin{1}='ms'; varargin{2}=5; varargin{3}='fms'; varargin{4}=5; noa=4; 
    k=1; figure(wnr); hold on; 
else
    wnr=varargin{1}; cmp=varargin{2}; noa=nargin-1; k=3;
    figure(wnr); hold on; 
end
xax=mat2str(p.nc.ilam(1)); yax=mat2str(cmp);
xlabel(char(['primary parameter = aux. var. ' xax])); 
if(cmp>0); ylabel(char(['user branch comp. ' yax]));
elseif(cmp==0); ylabel('L2-norm of u');
else yax=length(bradat(p))+p.plot.bpcmp; ylabel(char(['branch comp. ' yax]));
end    
pp=p.branch(length(bradat(p))+cmp,:); % the user data
if(noa==0); plot(p.branch(4,:),pp); return; end 
% standard values for varargin: 
ms=5;rms=0;fms=0;lms=5; % marker sizes for: bif-points, fold-points, reg.points, label-points 
inc=1; nl=0;  mnl=0;% inc for markers, length of label list, offsets for labels 
ox=0; oy=0; fs=16; lfs=fs; cl='black'; % offsets for labels, fontsize, color 
fp=1; lp=size(p.branch,2); % first and last point
lwst=4; lwun=2;  tyun='-'; tyst='-'; % line-widths and types stable and unstable 
while k<=noa 
    %k,varargin{k}
    switch lower(varargin{k})
      case 'lwun'; lwun=varargin{k+1}; k=k+2; % Linewidth of unstable Sol.
      case 'lwst'; lwst=varargin{k+1}; k=k+2; % Linewidth of stable Sol.
      case 'lw';   lwst=varargin{k+1}; lwun=lwst/2; k=k+2;
      case 'tyun'; tyun=varargin{k+1}; k=k+2; % Linetype of unstable Sol.
      case 'tyst'; tyst=varargin{k+1}; k=k+2; % Linetype of stable Sol.
      case 'ms';   ms=varargin{k+1}; k=k+2; % Markersize (bif.points)           
      case 'rms';  rms=varargin{k+1}; k=k+2; % Markersize (reg. points) 
      case 'fms';  fms=varargin{k+1}; k=k+2; % Markersize (fold points)           
      case 'lms';  lms=varargin{k+1}; k=k+2; % Markersize (labels) 
      case 'inc';  inc=varargin{k+1}; k=k+2; % increment for point markers
      case 'lab';  la=varargin{k+1}; nl=length(la); k=k+2; % label-list 
      case 'mlab'; mla=varargin{k+1}; mnl=length(mla); k=k+2; % label-list to be marked
      case 'labi'; bralen=size(p.branch,2); % labels with increment 
                   la=1:varargin{k+1}:bralen;
                   la=p.branch(1,la);
                   nl=size(la); nl=nl(2); k=k+2; 
      case 'labo'; ox=varargin{k+1};oy=varargin{k+2}; k=k+3; % label offsets 
      case 'fs';   fs=varargin{k+1}; k=k+2; % fontsize for labels 
      case 'lfs';  lfs=varargin{k+1}; k=k+2; % label fontsize for labels, if 0 no text
      case 'fp';   fp=varargin{k+1}; k=k+2; % first point
      case 'lp';   lp=varargin{k+1}; k=k+2; % last point
      case 'cl';   cl=varargin{k+1}; k=k+2; % color
      otherwise;   break; 
    end 
end 
for i=fp:lp-1 % loop from first-point to last point
    % linewidth is type of first point, unless this is a fold (2),
    % initial point (-1) (also from swibra (-2)) or branch point (1). In
    % this case line width is from second point.
    lw=lwun; lt=tyun; 
    if(~ismember(p.branch(4,i),[-2 -1 1 2]) && p.branch(3,i)  ==0) lw=lwst; lt=tyst; end
    if( ismember(p.branch(4,i),[-2 -1 1 2]) && p.branch(3,i+1)==0) lw=lwst; lt=tyst; end
    plot(p.branch(4,i:i+1),pp(i:i+1),lt,'Linewidth',lw,'color',cl);
    if(ms>0 && ismember(p.branch(2,i),[-2 1])) % plot markers for bif-points
      plot(p.branch(4,i),p.branch(length(bradat(p))+cmp,i),'o','MarkerSize',ms*lwun,'color',cl); 
    end
    if(fms>0 && ismember(p.branch(2,i),[2])) % plot markers for folds
      plot(p.branch(4,i),p.branch(length(bradat(p))+cmp,i),'x','MarkerSize',fms*lwun,'color',cl); 
    end
end
if(rms>0) % plot markers for regular points 
  for i=fp:inc:lp
    if (p.branch(3,i)==0) plot(p.branch(4,i),pp(i),'*','MarkerSize',rms,'color',cl);
    else plot(p.branch(4,i),pp(i),'+','MarkerSize',rms,'color',cl); % unstable point
    end
  end
end

for k=1:nl; % plot labels 
    if (k>=fp && k<=lp) 
      ls=mat2str(la(k)); 
      idx=find(p.branch(1,:)==la(k)); idx=idx(1);
      x=p.branch(4,idx)+ox; 
      y=pp(idx)+oy;
      if(lfs>0) text(x,y,ls,'FontSize',lfs,'color',cl); end;
      xx=p.branch(4,idx); yy=pp(idx);
      plot(xx,yy,'.','MarkerSize',3*lms*lwun,'color',cl);
    end
end 
axis tight;box on; set(gca,'FontSize',fs); 
