%%
% SPCALC: calculate p.nc.neig EVals of Gu closest to 0.
%
%  [ineg,muv]=spcalc(Gu,p)
%  [ineg,muv,V]=spcalc(Gu,p) - also compute and return eigenvectors in V
% 
% * muv=  eigenvalues sorted wrt real parts,
% * ineg= number of negative evals.
%
% See also eigs
function [ineg,muv,V]=spcalc(Gu,p)
M=getM(p); 
if(p.nc.nq>0) % trivially extend M in case of auxiliary equations
    M=[[M zeros(p.nu,p.nc.nq)]; [zeros(p.nc.nq,p.nu) speye(p.nc.nq)]]; 
end
if(strcmp(p.sw.eigmeth,'eigs')) % compute p.nc.neig FEM eigenvalues 
    vs=size(Gu,1); if p.sw.eigsstart==1; p.sw.evopts.v0=ones(vs,1)/vs; end 
    [V,mu]=eigs(Gu,M,min([p.nc.neig;vs-2]),p.nc.eigref,p.sw.evopts);
    muv=mu*ones(1,min([p.nc.neig;vs-2]))'; 
else % compute Evals with real-part in [p.nc.eigint(1), p.nc.eigint(2)], usually slow 
    evalc('[V,muv]=sptarn(Gu,M,p.nc.eigint(1),p.nc.eigint(2));');
end
mul=length(muv); [ms,ix]=sort(real(muv)); Vs=zeros(vs,mul);
ineg=0; mus=zeros(1,mul); impar=0;
for i=1:mul; mus(i)=muv(ix(i)); Vs(:,i)=V(:,ix(i));
    if (ms(i)<0) ineg=ineg+1; impar=impar+imag(mus(i)); end
end; 
if abs(mus(1))>abs(mus(mul))/2
    fprintf('warning: mu-min=%g, mu-max=%g; consider increasing p.nc.neig\n', mus(1),mus(mul));
end
if(impar>10*eps) 
    fprintf('warning: nonzero sum of imaginary parts of negative evals of Gu.\n');
end
muv=mus; V=Vs;
