%%
% SPCONTEXIT: exit spectral continuation; p.sw.spcont~=0 is required
%
%  q=spcontexit(dir,fname,inewpar)
%  q=spcontexit(dir,fname,inewpar,newdir)    - set problem name to newdir
%  q=spcontexit(dir,fname,inewpar,newdir,ds) - also set stepsize to ds
%
% * dir/fname = location of file with problem p-struct
% * inewpar=additional parameter required for additional equation solves
%
% See also spcontini
function q=spcontexit(dir,fname,varargin)
ffname=[dir '/' fname]; s=load(ffname,'p'); p=s.p;
if(p.sw.spcont==0) fprintf('Not a fold or branch point continuation problem.\n'); q=p; return; end
p.nu=p.nu/2; p.nc.neq=p.nc.neq/2; p.nc.nq=(p.nc.nq-1)/2; % set regular case sizes
p.sol.ptype=p.sw.spcont; p.sw.spcont=0;
p.u=[p.u(1:p.nu);p.u(2*p.nu+1:2*p.nu+p.naux)]; % non-fold/branch point variables
p.nc.ilam=p.nc.ilam(2:end); % remove current primary parameter
p.tau=[p.tau(1:p.nu);p.tau(2*p.nu+1:2*p.nu+p.nc.nq);0];
fprintf('Exiting fold/branch point continuation. Now\n');
printaux(p,1);

%generate p.mat.fill, p.mat.drop (these are not saved in data files)
if(p.sw.bcper==3)
    [p.mat.fill, p.mat.drop, p.nu]=getTorOp(p);
elseif(p.sw.bcper==1 || p.sw.bcper==2)
    [p.mat.fill, p.mat.drop, p.nu]=getCylOp(p);
end
if(p.sw.sfem~=0 || p.sw.spcalc~=0) p=setfemops(p); end

% other data initialization
p.file.bcount=1; p.file.count=1;
brout=p.fuha.outfu(p,p.u); % userfu to append to bif-branches
p.branch=[bradat(p); brout]; % put on branch
if(~isempty(varargin)); [p,ok]=setfn(p,varargin{1}); if ok~=1; q=p; return; end
else fprintf('warning: problem directory unchanged.\n'); end
%p.branch=bs(1:length(bradat(p))+length(p.fuha.outfu(p,p.u))); % cut off branch to new length
p.sol.deta=0; p.sol.ineg=-1;
fname=[p.file.pname,sprintf('%i',p.file.count),'.mat'];save(fname,'p');
p.file.count=2;p.file.bcount=1;q=p;
end